###I###R###D######U###2###3###3#######T###R###A###N###S###V###I###H###M###I####
###                                                                         ###
###           /\  ______      ____    _____     ___     ______  /\          ###
###           ||  \ \ \ \    (  _ \  (  _  )   / __)   / / / /  ||          ###
###           ||   > > > >    )   /   )(_)(   ( (__   < < < <   ||          ###
###           ||  /_/_/_/    (_)\_)  (___/\\   \___)   \_\_\_\  ||          ###
###           \/                                                \/          ###
###                                                                         ###
###I###R###D######U###2###3###3#######T###R###A###N###S###V###I###H###M###I####
# Name ___________________ indexing_genomes.smk
# Version ________________ v.2023.06
# Author _________________ Nicolas Fernandez
# Affiliation ____________ IRD_U233_TransVIHMI
# Aim ____________________ Snakefile with indexing genomes rules
# Date ___________________ 2022.09.28
# Latest modifications ___ 2023.06.21
# Use ____________________ snakemake -s indexing_genomes.smk --use-conda 

###############################################################################
### CONFIGURATION ###
#####################

configfile: "configuration/config.yaml"

###############################################################################
### FUNCTIONS ###
#################


###############################################################################
### WILDCARDS ###
#################

REF_SEQ, = glob_wildcards("resources/genomes/{ref_seq}.fasta")

###############################################################################
### RESOURCES ###
#################

OS = config["os"]                  # Operating system
CPUS = config["resources"]["cpus"] # Threads (maximum)

###############################################################################
### ENVIRONMENTS ###
####################

BWA = config["conda"][OS]["bwa"]         # BWA conda env

###############################################################################
### PARAMETERS ###
##################


###############################################################################
### RULES ###
#############

rule all:
    input:
        bwa_indexes = expand("resources/indexes/bwa/{ref_seq}.{ext}",
                            ref_seq = REF_SEQ,
                            ext = ["amb", "ann", "bwt", "pac", "sa"])
        
###############################################################################
rule bwa_genome_indexing:
    # Aim: index sequences in the FASTA format
    # Use: bwa index -p [PREFIX] [GENOME.fasta]
    message:
        "BWA-SW indexing {wildcards.ref_seq} genome"
    conda:
        BWA
    input:
        fasta = "resources/genomes/{ref_seq}.fasta"
    output:
        prefix = temp("resources/indexes/bwa/{ref_seq}"),
        indexes = multiext("resources/indexes/bwa/{ref_seq}",
                          ".amb", ".ann", ".bwt", ".pac", ".sa")
    log:
        "results/10_Reports/tools-log/bwa-indexes/{ref_seq}.log"
    shell:
        "bwa index "              # BWA-SW algorithm, index sequences
        "-p {output.prefix} "      # -p: Prefix of the output database
        "{input.fasta} "           # Reference sequences in the FASTA format
        "&> {log} "                # Log redirection
        "&& touch {output.prefix}" # Touch done

###############################################################################
